package com.example.proplayer;

import android.annotation.SuppressLint;
import android.content.ContentUris;
import android.content.Context;
import android.content.Intent;
import android.media.MediaMetadataRetriever;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.provider.MediaStore;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.PopupMenu;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.model.Model;
import com.google.android.material.snackbar.Snackbar;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

public class AudioAdapter extends RecyclerView.Adapter<AudioAdapter.MyViewHolder> {

    public Context aContext;
    static ArrayList<Audios> audiosFiles;

    AudioAdapter(Context aContext, ArrayList<Audios> audiosFiles){
        this.aContext = aContext;
        this.audiosFiles = audiosFiles;
    }

    @NonNull
    @Override
    public MyViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(aContext).inflate(R.layout.audio_items, parent, false);
        return new MyViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull MyViewHolder holder, @SuppressLint("RecyclerView") int position) {
        holder.file_name.setText(audiosFiles.get(position).getTitle());

        Glide.with(aContext)
                .load(R.drawable.play)
                .into(holder.audio_art);

        holder.itemView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent intent = new Intent(aContext, PlayerActivity.class);
                intent.putExtra("position", position);
                aContext.startActivity(intent);
            }
        });
        /*holder.delete_button.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(final View v) {
                PopupMenu popupMenu = new PopupMenu(aContext, v);
                popupMenu.getMenuInflater().inflate(R.menu.popup, popupMenu.getMenu());
                popupMenu.show();
                popupMenu.setOnMenuItemClickListener(new PopupMenu.OnMenuItemClickListener() {
                    @Override
                    public boolean onMenuItemClick(MenuItem item) {
                        switch (item.getItemId()) {
                            case R.id.delete:
                                //Toast.makeText(aContext, "Delete Clicked!", Toast.LENGTH_SHORT).show();
                                deleteAudioFile(position, v);
                                audiosFiles.remove(position);
                                break;
                        }
                        return true;
                    }
                });
            }
        });*/
    }

    private void deleteAudioFile(int position, View v) {
        Uri contentUri = ContentUris.withAppendedId(Uri.parse(String.valueOf(MediaStore.Audio.Media.EXTERNAL_CONTENT_URI)),
                Long.parseLong(audiosFiles.get(position).getId()));
        File file = new File(audiosFiles.get(position).getPath());
        boolean deleted = file.delete();
            if (deleted) {
                aContext.getApplicationContext().getContentResolver().delete(contentUri, null, null);
                audiosFiles.remove(position);
                notifyItemRemoved(position);
                notifyItemRangeChanged(position, audiosFiles.size());
                Snackbar.make(v, "File Deleted", Snackbar.LENGTH_SHORT).show();
            } else {
                Snackbar.make(v, "Cannot delete file", Snackbar.LENGTH_SHORT).show();
            }
    }

    @Override
    public int getItemCount() {
        return audiosFiles.size();
    }

    public class MyViewHolder extends RecyclerView.ViewHolder{

        TextView file_name;
        ImageView audio_art, delete_button;
        public MyViewHolder(@NonNull View itemView) {
            super(itemView);
            file_name = itemView.findViewById(R.id.audio_file_name);
            audio_art = itemView.findViewById(R.id.audio_img);
            //delete_button = itemView.findViewById(R.id.delete_button);
        }
    }

    void updateList(ArrayList<Audios> audiosArrayList){
        audiosFiles = new ArrayList<>();
        audiosFiles.addAll(audiosArrayList);
        this.notifyDataSetChanged();
    }

    /*private byte[] getAudioArt(String uri) throws IOException {

        MediaMetadataRetriever retriever = new MediaMetadataRetriever();
        if (Build.VERSION.SDK_INT >= 14){
            try {
                retriever.setDataSource(uri, new HashMap<String, String>());
            } catch (RuntimeException ex) {
                // something went wrong with the file, ignore it and continue
            }
        }
        else {
            retriever.setDataSource(uri);
        }
        byte[] art = retriever.getEmbeddedPicture();
        retriever.release();
        return art;
    }*/
}
