package com.example.proplayer;

import static android.app.PendingIntent.FLAG_MUTABLE;
import static com.example.proplayer.ApplicationClass.ACTION_NEXT;
import static com.example.proplayer.ApplicationClass.ACTION_PLAY;
import static com.example.proplayer.ApplicationClass.ACTION_PREVIOUS;
import static com.example.proplayer.ApplicationClass.CHANNEL_ID_2;
import static com.example.proplayer.PlayerActivity.audioList;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.media.MediaPlayer;
import android.net.Uri;
import android.os.Binder;
import android.os.Build;
import android.os.IBinder;
import android.support.v4.media.session.MediaSessionCompat;
import android.util.Log;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import androidx.core.app.NotificationCompat;

import java.util.ArrayList;

public class AudioService extends Service implements MediaPlayer.OnCompletionListener {

    IBinder aBinder = new MyBinder();
    Uri uri;
    int position = 0;
    MediaPlayer mediaPlayer;
    ArrayList<Audios> audioFileArray = new ArrayList<>();
    ActionPlaying actionPlaying;
    MediaSessionCompat mediaSessionCompat;
    public static final String LAST_AUDIO_PLAYED = "LAST_PLAYED";
    public static final String AUDIO_FILE = "STORED_AUDIO";
    public static final String ARTIST_NAME = "ARTIST NAME";
    public static final String AUDIO_NAME = "AUDIO NAME";



    @Override
    public void onCreate() {
        super.onCreate();
        mediaSessionCompat = new MediaSessionCompat(getApplicationContext(), "My Audio");
    }

    public class MyBinder extends Binder {
        AudioService getService(){
            return AudioService.this;
        }
    }

    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        Log.e("Bind", "Method");
        return aBinder;
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        int myPosition = intent.getIntExtra("servicePosition", -1);
        String actionName = intent.getStringExtra("ActionName");

        if(myPosition != -1){
            playMedia(myPosition);
        }

        if(actionName != null){
            switch (actionName){
                case "playPause":
                    //Toast.makeText(this, "PlayPause", Toast.LENGTH_SHORT).show();
                    playPauseButtonClicked();
                    break;
                case "next":
                    //Toast.makeText(this, "Next", Toast.LENGTH_SHORT).show();
                    nextButtonClicked();
                    break;
                case "previous":
                    //Toast.makeText(this, "Previous", Toast.LENGTH_SHORT).show();
                    previousButtonClicked();
                    break;
            }
        }
        return START_STICKY;
    }

    private void playMedia(int startPosition) {
        audioFileArray = audioList;
        position = startPosition;
        if(mediaPlayer != null){
            mediaPlayer.stop();
            mediaPlayer.release();
            if(audioFileArray != null){
                createMediaPlayer(position);
                mediaPlayer.start();
            }
        }else{
            createMediaPlayer(position);
            mediaPlayer.start();
        }
    }

    void start(){
        mediaPlayer.start();
    }

    boolean isPlaying(){
        if(mediaPlayer != null){
            return mediaPlayer.isPlaying();
        }else{
            Toast.makeText(this, "Please, select an audio first", Toast.LENGTH_SHORT).show();
            return false;
        }
    }

    void stop(){
        mediaPlayer.stop();
    }

    void release(){
        mediaPlayer.release();
    }

    int getDuration(){
       return mediaPlayer.getDuration();
    }

    void seekTo(int position){
        mediaPlayer.seekTo(position);
    }

    int getCurrentPosition(){
        return mediaPlayer.getCurrentPosition();
    }

    void pause(){
        mediaPlayer.pause();
    }

    void createMediaPlayer(int positionInner){
        position = positionInner;
        uri = Uri.parse(audioFileArray.get(position).getPath());
        SharedPreferences.Editor editor = getSharedPreferences(LAST_AUDIO_PLAYED, MODE_PRIVATE).edit();
        editor.putString(AUDIO_FILE, uri.toString());
        editor.putString(ARTIST_NAME, audioList.get(position).getArtist());
        editor.putString(AUDIO_NAME, audioList.get(position).getTitle());
        editor.apply();
        mediaPlayer = MediaPlayer.create(getBaseContext(), uri);
    }

    void onCompleted (){
        mediaPlayer.setOnCompletionListener(this);
    }

    @Override
    public void onCompletion(MediaPlayer mp) {
        if(actionPlaying != null){
            actionPlaying.nextButtonClicked();
            if(mediaPlayer != null){
                createMediaPlayer(position);
                mediaPlayer.start();
                onCompleted();
            }
        }
    }

    void setCallBack(ActionPlaying actionPlaying){
        this.actionPlaying = actionPlaying;
    }

    @RequiresApi(api = Build.VERSION_CODES.S)
    void showNotification(int playPauseButton){
        Intent intent = new Intent(this, PlayerActivity.class);
        PendingIntent contentIntent = PendingIntent.getActivity(this, 0, intent, PendingIntent.FLAG_MUTABLE);

        Intent previousIntent = new Intent(this, NotificationReceiver.class).setAction(ACTION_PREVIOUS);
        PendingIntent previousPending = PendingIntent.getBroadcast(this, 0, previousIntent, PendingIntent.FLAG_MUTABLE);

        Intent pauseIntent = new Intent(this, NotificationReceiver.class).setAction(ACTION_PLAY);
        PendingIntent pausePending = PendingIntent.getBroadcast(this, 0, pauseIntent, PendingIntent.FLAG_MUTABLE);

        Intent nextIntent = new Intent(this, NotificationReceiver.class).setAction(ACTION_NEXT);
        PendingIntent nextPending = PendingIntent.getBroadcast(this, 0, nextIntent, PendingIntent.FLAG_MUTABLE);

        Bitmap thumbnail;

        thumbnail = BitmapFactory.decodeResource(getResources(), R.drawable.logo);

        Notification notification = new NotificationCompat.Builder(this, CHANNEL_ID_2)
                .setSmallIcon(playPauseButton)
                .setLargeIcon(thumbnail)
                .setContentTitle(audioFileArray.get(position).getTitle())
                .setContentText(audioFileArray.get(position).getArtist())
                .addAction(R.drawable.skip_previous, "Previous", previousPending)
                .addAction(playPauseButton, "Pause", pausePending)
                .addAction(R.drawable.skip_next, "Next", nextPending)
                .setStyle(new androidx.media.app.NotificationCompat.MediaStyle().setMediaSession(mediaSessionCompat.getSessionToken()))
                .setPriority(NotificationCompat.PRIORITY_HIGH)
                .setOnlyAlertOnce(true)
                .setVisibility(NotificationCompat.VISIBILITY_PUBLIC)
                .build();

        startForeground(2, notification);
    }

    void playPauseButtonClicked (){
        if(actionPlaying != null){
            actionPlaying.playPauseButtonClicked();
        }
    }

    boolean nextButtonClicked (){
        if(actionPlaying != null){
            actionPlaying.nextButtonClicked();
            return true;
        }else{
            Toast.makeText(this, "Please, select an audio first", Toast.LENGTH_SHORT).show();
            return false;
        }
    }

    void previousButtonClicked (){
        if(actionPlaying != null){
            actionPlaying.prevButtonClicked();
        }
    }
}
