package com.example.proplayer;

import static com.example.proplayer.AudiosFragment.audioAdapter;


import android.Manifest;
import android.annotation.SuppressLint;
import android.app.DownloadManager;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.provider.Settings;
import android.util.SparseArray;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.SearchView;
import android.widget.Toast;


import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.viewpager.widget.ViewPager;

import com.google.android.material.tabs.TabLayout;

import java.io.File;
import java.util.ArrayList;

import at.huber.youtubeExtractor.VideoMeta;
import at.huber.youtubeExtractor.YouTubeExtractor;
import at.huber.youtubeExtractor.YouTubeUriExtractor;
import at.huber.youtubeExtractor.YtFile;

public class MainActivity extends AppCompatActivity implements SearchView.OnQueryTextListener {

    public static final int REQUEST_CODE = 1;
    static ArrayList<Audios> audios;
    static boolean shuffleButtonBoolean = false, repeatButtonBoolean = false;
    private String MY_SORT_PREFERENCE = "SortOrder";
    private EditText editText;
    String newLink;
    public static final String LAST_AUDIO_PLAYED = "LAST_PLAYED";
    public static final String AUDIO_FILE = "STORED_AUDIO";
    static boolean SHOW_MINI_PLAYER = false;
    public static String PATH_TO_FRAGMENT = null;

    public static final String ARTIST_NAME = "ARTIST NAME";
    public static final String AUDIO_NAME = "AUDIO NAME";

    public static String ARTIST_TO_FRAGMENT = null;
    public static String AUDIO_TO_FRAGMENT = null;

    @SuppressLint({"MissingInflatedId", "WrongViewCast"})
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        try {
            Thread.sleep(1000);
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
        setTheme(R.style.SplashTheme);
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            permission();
        }
        editText = findViewById(R.id.youtube_link);
    }
    public void DownloadMyVideo(View view){
        editText = findViewById(R.id.youtube_link);
        String values = editText.getText().toString();

        if(values.equals("")){
            Toast.makeText(this, "Type your YouTube link first", Toast.LENGTH_SHORT).show();
        }else if (!values.startsWith("https://youtu")){
            Toast.makeText(this, "Type a valid YouTube link", Toast.LENGTH_SHORT).show();
        }else {
            YouTubeExtractor youTubeExtractor = new YouTubeUriExtractor(MainActivity.this) {
                @Override
                public void onUrisAvailable(String videoId, String videoTitle, SparseArray<YtFile> ytFiles) {
                    if (ytFiles != null) {
                        int tag = 140;
                        newLink = ytFiles.get(tag).getUrl();
                        DownloadManager.Request request = new DownloadManager.Request(Uri.parse(newLink));
                        request.setTitle(videoTitle);
                        request.setDestinationInExternalPublicDir(Environment.DIRECTORY_DOWNLOADS, videoTitle + ".mp3");
                        DownloadManager downloadManager = (DownloadManager) getSystemService(Context.DOWNLOAD_SERVICE);
                        request.allowScanningByMediaScanner();
                        request.setAllowedNetworkTypes(DownloadManager.Request.NETWORK_MOBILE | DownloadManager.Request.NETWORK_WIFI);
                        downloadManager.enqueue(request);
                    }
                }
            };

            youTubeExtractor.execute(values);
            Toast.makeText(this, "Your download will start soon, please wait", Toast.LENGTH_SHORT).show();
        }
    }

    @RequiresApi(api = Build.VERSION_CODES.R)
    private void permission() {

        if(ContextCompat.checkSelfPermission(getApplicationContext(), Manifest.permission.MANAGE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED){
            ActivityCompat.requestPermissions(MainActivity.this, new String[]{Manifest.permission.MANAGE_EXTERNAL_STORAGE}, REQUEST_CODE);
        }
        audios = getAllAudios(this);
        initViewPager();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if(requestCode == REQUEST_CODE){
            if(grantResults[0] == PackageManager.PERMISSION_GRANTED){
                audios = getAllAudios(this);
                initViewPager();
            }else{
                ActivityCompat.requestPermissions(MainActivity.this, new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, REQUEST_CODE);
            }
        }
    }

    private void initViewPager() {
        ViewPager viewPager = findViewById(R.id.viewpager);
        TabLayout tabLayout = findViewById(R.id.tab_layout);
        ViewPagerAdapter viewPagerAdapter = new ViewPagerAdapter(getSupportFragmentManager());
        viewPagerAdapter.addFragments(new AudiosFragment(), "Audios");
        viewPagerAdapter.addFragments(new DownloadFragment(), "Download");
        viewPager.setAdapter(viewPagerAdapter);
        tabLayout.setupWithViewPager(viewPager);
    }

    public static class ViewPagerAdapter extends FragmentPagerAdapter{

        private ArrayList<Fragment> fragments;
        private ArrayList<String> titles;

        public ViewPagerAdapter(@NonNull FragmentManager fm) {
            super(fm);
            this.fragments = new ArrayList<>();
            this.titles = new ArrayList<>();
        }

        void addFragments(Fragment fragment, String title){
            fragments.add(fragment);
            titles.add(title);
        }

        @NonNull
        @Override
        public Fragment getItem(int position) {
            return fragments.get(position);
        }

        @Override
        public int getCount() {
            return fragments.size();
        }

        @Nullable
        @Override
        public CharSequence getPageTitle(int position) {
            return titles.get(position);
        }
    }

    public ArrayList<Audios> getAllAudios(Context context){
            SharedPreferences preferences = getSharedPreferences(MY_SORT_PREFERENCE, MODE_PRIVATE);
            String sortOrder = preferences.getString("sorting", "sortByName");

            ArrayList<Audios> aux = new ArrayList<>();

            String order = null;

            Uri uri = MediaStore.Audio.Media.EXTERNAL_CONTENT_URI;
            switch (sortOrder){
                case "sortByName":
                    order = MediaStore.MediaColumns.DISPLAY_NAME  + " ASC";
                    break;
                case "sortBySize":
                    order = MediaStore.MediaColumns.SIZE  + " DESC";
                    break;
                case "sortByDate":
                    order = MediaStore.MediaColumns.DATE_ADDED  + " DESC";
                    break;
            }

            String[] projection = {
                    MediaStore.Audio.Media.ALBUM,
                    MediaStore.Audio.Media.TITLE,
                    MediaStore.Audio.Media.DURATION,
                    MediaStore.Audio.Media.DATA,
                    MediaStore.Audio.Media.ARTIST,
                    MediaStore.Audio.Media._ID
            };
            Cursor cursor = context.getContentResolver().query(uri, projection, null, null, order);

            if(cursor != null){
                while(cursor.moveToNext()){
                    String album = cursor.getString(0);
                    String title = cursor.getString(1);
                    String duration = cursor.getString(2);
                    String path = cursor.getString(3);
                    String artist = cursor.getString(4);
                    String id = cursor.getString(5);

                    Audios audios = new Audios(path, title, artist, album, duration, id);
                    aux.add(audios);
                }
                cursor.close();
            }
            return aux;
        }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.search, menu);
        MenuItem menuItem = menu.findItem(R.id.search_option);
        SearchView searchView = (SearchView) menuItem.getActionView();
        searchView.setOnQueryTextListener(this);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onQueryTextSubmit(String s) {
        return false;
    }

    @Override
    public boolean onQueryTextChange(String input) {
        String userInput = input.toLowerCase();
        ArrayList<Audios> myAudios = new ArrayList<>();
        for (Audios audio: audios) {
           if(audio.getTitle().toLowerCase().contains(userInput)){
                myAudios.add(audio);
           }
        }
        audioAdapter.updateList(myAudios);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        SharedPreferences.Editor editor = getSharedPreferences(MY_SORT_PREFERENCE, MODE_PRIVATE).edit();
        switch (item.getItemId()){
            case R.id.sort_by_name:
                editor.putString("sorting", "sortByName");
                editor.apply();
                this.recreate();
                break;
            case R.id.sort_by_size:
                editor.putString("sorting", "sortBySize");
                editor.apply();
                this.recreate();
                break;
            case R.id.sort_by_date:
                editor.putString("sorting", "sortByDate");
                editor.apply();
                this.recreate();
                break;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onResume() {
        super.onResume();
        SharedPreferences preferences = getSharedPreferences(LAST_AUDIO_PLAYED, MODE_PRIVATE);
        String path = preferences.getString(AUDIO_FILE, null);
        String artist = preferences.getString(ARTIST_NAME, null);
        String audioName = preferences.getString(AUDIO_NAME, null);

        if(path != null){
            SHOW_MINI_PLAYER = true;
            PATH_TO_FRAGMENT = path;
            ARTIST_TO_FRAGMENT = artist;
            AUDIO_TO_FRAGMENT = audioName;
        }else{
            SHOW_MINI_PLAYER = false;
            PATH_TO_FRAGMENT = null;
            ARTIST_TO_FRAGMENT = null;
            AUDIO_TO_FRAGMENT = null;
        }
    }
}