package com.example.proplayer;

import static android.content.Context.MODE_PRIVATE;
import static com.example.proplayer.MainActivity.ARTIST_TO_FRAGMENT;
import static com.example.proplayer.MainActivity.AUDIO_TO_FRAGMENT;
import static com.example.proplayer.MainActivity.PATH_TO_FRAGMENT;
import static com.example.proplayer.MainActivity.SHOW_MINI_PLAYER;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.IBinder;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.fragment.app.Fragment;

import com.bumptech.glide.Glide;
import com.google.android.material.floatingactionbutton.FloatingActionButton;

import java.util.Objects;


public class NowPlayingFragment extends Fragment  implements ServiceConnection{

    ImageView nextButton, albumArt;
    TextView artist, audioName;
    FloatingActionButton playPauseButtonMini;
    View view;
    AudioService audioService;

    public static final String LAST_AUDIO_PLAYED = "LAST_PLAYED";
    public static final String AUDIO_FILE = "STORED_AUDIO";
    public static final String ARTIST_NAME = "ARTIST NAME";
    public static final String AUDIO_NAME = "AUDIO NAME";

    public NowPlayingFragment() { }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        view = inflater.inflate(R.layout.fragment_now_playing, container, false);
        artist = view.findViewById(R.id.audio_artist_mini_player);
        audioName = view.findViewById(R.id.audio_name_mini_player);
        nextButton = view.findViewById(R.id.skip_next_botton);
        albumArt = view.findViewById(R.id.bottom_album_art);
        playPauseButtonMini = view.findViewById(R.id.play_pause_mini_player);

        nextButton.setOnClickListener(view -> {
            if(audioService != null){
                if(audioService.nextButtonClicked()) {
                    if (getActivity() != null) {
                        SharedPreferences.Editor editor = getActivity().getSharedPreferences(LAST_AUDIO_PLAYED, MODE_PRIVATE).edit();
                        editor.putString(AUDIO_FILE, audioService.audioFileArray.get(audioService.position).getPath());
                        editor.putString(ARTIST_NAME, audioService.audioFileArray.get(audioService.position).getArtist());
                        editor.putString(AUDIO_NAME, audioService.audioFileArray.get(audioService.position).getTitle());
                        editor.apply();
                        SharedPreferences preferences = getActivity().getSharedPreferences(LAST_AUDIO_PLAYED, MODE_PRIVATE);
                        String path = preferences.getString(AUDIO_FILE, null);
                        String artistName = preferences.getString(ARTIST_NAME, null);
                        String audioFileName = preferences.getString(AUDIO_NAME, null);

                        if (path != null) {
                            SHOW_MINI_PLAYER = true;
                            PATH_TO_FRAGMENT = path;
                            ARTIST_TO_FRAGMENT = artistName;
                            AUDIO_TO_FRAGMENT = audioFileName;
                        } else {
                            SHOW_MINI_PLAYER = false;
                            PATH_TO_FRAGMENT = null;
                            ARTIST_TO_FRAGMENT = null;
                            AUDIO_TO_FRAGMENT = null;
                        }
                        if (SHOW_MINI_PLAYER) {
                            Glide.with(requireContext()).load(R.drawable.play).into(albumArt);
                            audioName.setText(AUDIO_TO_FRAGMENT);
                            artist.setText(ARTIST_TO_FRAGMENT);
                        }
                    }
                }
            }
        });

        playPauseButtonMini.setOnClickListener(view -> {
            if (audioService == null){
                Toast.makeText(getContext(), "Play an audio First", Toast.LENGTH_SHORT).show();
            }else{
                audioService.playPauseButtonClicked();
                if(audioService.isPlaying()){
                    playPauseButtonMini.setImageResource(R.drawable.pause);
                }else{
                    playPauseButtonMini.setImageResource(R.drawable.play_arrow);
                }
            }
        });

        return view;
    }

    boolean aIsBound;
    public void doBindService() {
        aIsBound = requireContext().bindService(new Intent(getContext(), AudioService.class), this, Context.BIND_AUTO_CREATE);
    }

    public void undoBindService(){
        if(aIsBound){
            requireContext().unbindService(this);
            aIsBound = false;
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        if(SHOW_MINI_PLAYER){
            if(PATH_TO_FRAGMENT != null) {
                Glide.with(requireContext()).load(R.drawable.play).into(albumArt);
                audioName.setText(AUDIO_TO_FRAGMENT);
                artist.setText(ARTIST_TO_FRAGMENT);
            }
            doBindService();
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        undoBindService();
    }

    @Override
    public void onServiceConnected(ComponentName componentName, IBinder service) {
        AudioService.MyBinder binder = (AudioService.MyBinder) service;
        audioService = binder.getService();
    }

    @Override
    public void onServiceDisconnected(ComponentName componentName) {
        audioService = null;
    }
}