package com.example.proplayer;

import static com.example.proplayer.AudioAdapter.audiosFiles;
import static com.example.proplayer.MainActivity.repeatButtonBoolean;
import static com.example.proplayer.MainActivity.shuffleButtonBoolean;

import android.content.ComponentName;
import android.content.Intent;
import android.content.ServiceConnection;
import android.media.MediaMetadataRetriever;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;

import com.bumptech.glide.Glide;
import com.google.android.material.floatingactionbutton.FloatingActionButton;

import java.util.ArrayList;
import java.util.Random;

public class PlayerActivity extends AppCompatActivity implements ActionPlaying, ServiceConnection {

    TextView audio_name, artist_name, duration_played, duration_total;
    ImageView cover_art, nextButton, previousButton, backButton, shuffleButton, repeatButton;
    FloatingActionButton playPauseButton;
    SeekBar seekBar;
    int position = -1;
    static ArrayList<Audios> audioList = new ArrayList<>();
    static Uri uri;
    //static MediaPlayer mediaPlayer;
    private Handler handler = new Handler();
    private Thread playThread, prevThread, nextThread;
    AudioService audioService;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setFullScreen();
        setContentView(R.layout.activity_player);
        getSupportActionBar().hide();
        initView();
        getIntentMethod();
        seekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                if(audioService != null && fromUser){
                    audioService.seekTo(progress * 1000);
                }
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });
        PlayerActivity.this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if(audioService != null){
                    int aCurrentPosition = audioService.getCurrentPosition() / 1000;
                    seekBar.setProgress(aCurrentPosition);
                    duration_played.setText(formattedTime(aCurrentPosition));
                }
                handler.postDelayed(this, 1000);
            }
        });
        shuffleButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if(shuffleButtonBoolean){
                    shuffleButtonBoolean = false;
                    shuffleButton.setImageResource(R.drawable.shuffle_off);
                }else{
                    shuffleButtonBoolean = true;
                    shuffleButton.setImageResource(R.drawable.shuffle_on);
                }
            }
        });
        repeatButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if(repeatButtonBoolean){
                    repeatButtonBoolean = false;
                    repeatButton.setImageResource(R.drawable.repeat_off);
                }else{
                    repeatButtonBoolean = true;
                    repeatButton.setImageResource(R.drawable.repeat_on);
                }
            }
        });
    }

    private void setFullScreen() {
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
    }

    @Override
    protected void onResume() {
        Intent intent = new Intent(this, AudioService.class);
        bindService(intent, this, BIND_AUTO_CREATE);
        playThread();
        nextThread();
        prevThread();
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
        unbindService(this);
    }

    private void nextThread() {
        nextThread = new Thread()
        {
            @Override
            public void run(){
                super.run();
                nextButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        nextButtonClicked();
                    }
                });
            }
        };
        nextThread.start();
    }

    public boolean nextButtonClicked() {
        if(audioService.isPlaying()){
            audioService.stop();
            audioService.release();
            if(shuffleButtonBoolean && !repeatButtonBoolean){
                position = getRandom(audioList.size() - 1);
            } else if (!shuffleButtonBoolean && !repeatButtonBoolean) {
                position = ((position + 1) % audioList.size());
            }
            uri = Uri.parse(audioList.get(position).getPath());
            audioService.createMediaPlayer(position);
            metaData(uri);
            audio_name.setText(audioList.get(position).getTitle());
            artist_name.setText(audioList.get(position).getArtist());
            seekBar.setMax(audioService.getDuration()/1000);
            PlayerActivity.this.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    if(audioService != null){
                        int aCurrentPosition = audioService.getCurrentPosition() / 1000;
                        seekBar.setProgress(aCurrentPosition);
                    }
                    handler.postDelayed(this, 1000);
                }
            });
            audioService.onCompleted();
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                audioService.showNotification(R.drawable.pause);
            }
            playPauseButton.setBackgroundResource(R.drawable.pause);
            audioService.start();
            return true;
        }else{
            audioService.stop();
            audioService.release();
            if(shuffleButtonBoolean && !repeatButtonBoolean){
                position = getRandom(audioList.size() - 1);
            } else if (!shuffleButtonBoolean && !repeatButtonBoolean) {
                position = ((position + 1) % audioList.size());
            }
            uri = Uri.parse(audioList.get(position).getPath());
            audioService.createMediaPlayer(position);
            metaData(uri);
            audio_name.setText(audioList.get(position).getTitle());
            artist_name.setText(audioList.get(position).getArtist());
            seekBar.setMax(audioService.getDuration()/1000);
            PlayerActivity.this.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    if(audioService != null){
                        int aCurrentPosition = audioService.getCurrentPosition() / 1000;
                        seekBar.setProgress(aCurrentPosition);
                    }
                    handler.postDelayed(this, 1000);
                }
            });
            audioService.onCompleted();
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                audioService.showNotification(R.drawable.play_arrow);
            }
            playPauseButton.setBackgroundResource(R.drawable.play_arrow);
            return true;
        }
    }

    private int getRandom(int i) {
        Random random = new Random();
        return random.nextInt(i + 1);
    }

    private void prevThread() {
        prevThread = new Thread()
        {
            @Override
            public void run(){
                super.run();
                previousButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        prevButtonClicked();
                    }
                });
            }
        };
        prevThread.start();
    }

    public void prevButtonClicked() {
        if(audioService.isPlaying())
        {
            audioService.stop();
            audioService.release();
            if(shuffleButtonBoolean && !repeatButtonBoolean){
                position = getRandom(audioList.size() - 1);
            } else if (!shuffleButtonBoolean && !repeatButtonBoolean) {
                position = ((position - 1) < 0 ? (audioList.size() - 1) : (position - 1));
            }
            uri = Uri.parse(audioList.get(position).getPath());
            audioService.createMediaPlayer(position);
            metaData(uri);
            audio_name.setText(audioList.get(position).getTitle());
            artist_name.setText(audioList.get(position).getArtist());
            seekBar.setMax(audioService.getDuration()/1000);
            PlayerActivity.this.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    if(audioService != null){
                        int aCurrentPosition = audioService.getCurrentPosition() / 1000;
                        seekBar.setProgress(aCurrentPosition);
                    }
                    handler.postDelayed(this, 1000);
                }
            });
            audioService.onCompleted();
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                audioService.showNotification(R.drawable.pause);
            }
            playPauseButton.setBackgroundResource(R.drawable.pause);
            audioService.start();
        }else{
            audioService.stop();
            audioService.release();
            if(shuffleButtonBoolean && !repeatButtonBoolean){
                position = getRandom(audioList.size() - 1);
            } else if (!shuffleButtonBoolean && !repeatButtonBoolean) {
                position = ((position - 1) < 0 ? (audioList.size() - 1) : (position - 1));
            }
            uri = Uri.parse(audioList.get(position).getPath());
            audioService.createMediaPlayer(position);
            metaData(uri);
            audio_name.setText(audioList.get(position).getTitle());
            artist_name.setText(audioList.get(position).getArtist());
            seekBar.setMax(audioService.getDuration()/1000);
            PlayerActivity.this.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    if(audioService != null){
                        int aCurrentPosition = audioService.getCurrentPosition() / 1000;
                        seekBar.setProgress(aCurrentPosition);
                    }
                    handler.postDelayed(this, 1000);
                }
            });
            audioService.onCompleted();
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                audioService.showNotification(R.drawable.play_arrow);
            }
            playPauseButton.setBackgroundResource(R.drawable.play_arrow);
        }
    }

    private void playThread() {
        playThread = new Thread()
        {
            @Override
            public void run(){
                super.run();
                playPauseButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        playPauseButtonClicked();
                    }
                });
            }
        };
        playThread.start();
    }

    public void playPauseButtonClicked(){
        if(audioService.isPlaying()){
            playPauseButton.setImageResource(R.drawable.play_arrow);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                audioService.showNotification(R.drawable.play_arrow);
            }
            audioService.pause();
            seekBar.setMax(audioService.getDuration()/1000);
            PlayerActivity.this.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    if(audioService != null){
                        int aCurrentPosition = audioService.getCurrentPosition() / 1000;
                        seekBar.setProgress(aCurrentPosition);
                    }
                    handler.postDelayed(this, 1000);
                }
            });
        }else{
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                audioService.showNotification(R.drawable.pause);
            }
            playPauseButton.setImageResource(R.drawable.pause);
            audioService.start();
            seekBar.setMax(audioService.getDuration()/1000);
            PlayerActivity.this.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    if(audioService != null){
                        int aCurrentPosition = audioService.getCurrentPosition() / 1000;
                        seekBar.setProgress(aCurrentPosition);
                    }
                    handler.postDelayed(this, 1000);
                }
            });
        }
    }

    private String formattedTime(int aCurrentPosition) {
        String totalOut = "";
        String totalNew = "";
        String seconds = String.valueOf(aCurrentPosition % 60);
        String minutes = String.valueOf(aCurrentPosition / 60);
        totalOut = minutes + ":" + seconds;
        totalNew = minutes + ":" + "0" + seconds;
        if(seconds.length() == 1){
            return totalNew;
        }else{
            return totalOut;
        }
    }

    private void getIntentMethod() {
        position = getIntent().getIntExtra("position", -1);
        String sender = getIntent().getStringExtra("sender");
        audioList = audiosFiles;
        if (audioList != null) {
            playPauseButton.setImageResource(R.drawable.pause);
            uri = Uri.parse(audioList.get(position).getPath());
        }
        /*if(audioService != null){
            audioService.stop();
            audioService.release();
            audioService.createMediaPlayer(position);
            audioService.start();
        }else{
            if(uri != null){
                audioService.createMediaPlayer(position);
                audioService.start();
            }
        }*/

        Intent intent = new Intent(this, AudioService.class);
        intent.putExtra("servicePosition", position);
        startService(intent);
    }

    private void initView() {
        audio_name = findViewById(R.id.audio_name);
        artist_name = findViewById(R.id.song_artist);
        duration_played = findViewById(R.id.durationPlayed);
        duration_total = findViewById(R.id.durationTotal);
        cover_art = findViewById(R.id.cover_art);
        nextButton = findViewById(R.id.skip_next);
        previousButton = findViewById(R.id.skip_previous);
        backButton = findViewById(R.id.back_button);
        shuffleButton = findViewById(R.id.shuffle);
        repeatButton = findViewById(R.id.repeat_off);
        playPauseButton = findViewById(R.id.play_pause);
        seekBar = findViewById(R.id.seekBar);
    }

    private void metaData (Uri uri){
        MediaMetadataRetriever retriever = new MediaMetadataRetriever();
        retriever.setDataSource(uri.toString());
        int durationTotal = Integer.parseInt(audioList.get(position).getDuration()) / 1000;
        duration_total.setText(formattedTime(durationTotal));
        byte[] art = retriever.getEmbeddedPicture();
        if(art != null){
            Glide.with(this).asBitmap().load(art).into(cover_art);
        }else{
            //Glide.with(this).asBitmap().load(R.drawable.logo).into(cover_art);
        }
    }

    /*@Override
    public void onCompletion(MediaPlayer mp) {
        nextButtonClicked();
        if(audioService != null){
            audioService.createMediaPlayer(position);
            audioService.start();
            audioService.onCompleted();
        }
    }*/

    @Override
    public void onServiceConnected(ComponentName componentName, IBinder service) {
        AudioService.MyBinder myBinder = (AudioService.MyBinder) service;
        audioService = myBinder.getService();
        audioService.setCallBack(this);
        //Toast.makeText(this, "Connected" + audioService, Toast.LENGTH_SHORT).show();
        seekBar.setMax(audioService.getDuration()/1000);
        metaData(uri);
        audio_name.setText(audioList.get(position).getTitle());
        artist_name.setText(audioList.get(position).getArtist());
        audioService.onCompleted();
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
            audioService.showNotification(R.drawable.pause);
        }
    }

    @Override
    public void onServiceDisconnected(ComponentName componentName) {
        audioService = null;
    }

}